# Generated by Django 2.2.6 on 2019-10-23 14:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oauth_clients', '0005_clientparam'),
    ]

    operations = [
        migrations.AddField(
            model_name='accesstoken',
            name='scope',
            field=models.TextField(blank=True, default='', verbose_name='Scope'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-11-21 20:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oauth_clients', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='client',
            name='scope',
            field=models.CharField(blank=True, default=b'read', max_length=255, verbose_name='Scope'),
        ),
    ]



# Generated by Django 2.2.28 on 2025-10-13 06:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oauth_clients', '0006_accesstoken_scope'),
    ]

    operations = [
        migrations.AlterField(
            model_name='accesstoken',
            name='access_token',
            field=models.TextField(verbose_name='Access token'),
        ),
        migrations.AlterField(
            model_name='accesstoken',
            name='refresh_token',
            field=models.TextField(verbose_name='Refresh token'),
        ),
    ]



from django.contrib import admin
try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse
from django.utils.translation import ugettext_lazy as _, ugettext
from django.utils.formats import localize
from django.conf.urls import url
from django.http import HttpResponse
from django.shortcuts import get_object_or_404, redirect
from django.contrib import messages
from . import models


class TokenInline(admin.TabularInline):
    fields = ('user_id', 'username', 'token_type', 'expires_in', 'access_token', 'refresh_token')
    readonly_fields = fields
    extra = 0
    model = models.AccessToken

class ParamInline(admin.TabularInline):
    fields = ('name', 'value')
    extra = 2
    model = models.ClientParam

class ClientAdmin(admin.ModelAdmin):
    list_display = ('uid', 'name', 'client_id', 'authorization_endpoint', 'token_endpoint',
        'scope', 'creator', 'created', 'modified')
    fields = ('uid', 'name', 'client_id', 'client_secret', 'authorization_endpoint',
        'token_endpoint', 'creator', 'scope', 'created', 'modified', 'complete_authorization_url')
    readonly_fields = ('uid', 'creator', 'created', 'modified', 'complete_authorization_url')
    inlines = [ParamInline, TokenInline]

    def get_urls(self):
        urls = super(ClientAdmin, self).get_urls()
        info = self.model._meta.app_label, self.model._meta.model_name
        my_urls = [
            url(r'^(.+)/complete-authorization/$', self.complete_authorization, name='%s_%s_complete_authorization' % info),
        ]
        return my_urls + urls

    def complete_authorization(self, request, uid):
        "View to complete oauth2 authorization"
        client = get_object_or_404(models.Client, uid=uid)
        redirect_url = client.complete_authorization_url(request)
        final_redirection = client.complete_authorization(request, redirect_url)
        msg = ugettext("Oauth client authorized.")
        self.message_user(request, msg, messages.INFO)
        if final_redirection:
            return redirect(final_redirection)
        url_name = 'admin:{}_{}_change'.format(self.model._meta.app_label, self.model._meta.model_name)
        return redirect(url_name, client.pk)

    def save_model(self, request, obj, form, change):
        if obj.pk is None:
            obj.creator = request.user
        super(ClientAdmin, self).save_model(request, obj, form, change)

    def response_change(self, request, obj):
        if '_authorize-client' in request.POST:
            redirect_url = obj.complete_authorization_url(request)
            authorization_url = obj.start_authorization_url(request, redirect_url)
            return redirect(authorization_url)
        return super(ClientAdmin, self).response_change(request, obj)


class TokenAdmin(admin.ModelAdmin):
    list_display = ('client', 'user_id', 'username', 'token_type', 'get_expiration',
        'get_expired', 'modified')
    fields = ('client', 'user_id', 'username', 'token_type', 'scope', 'expires_in',
        'get_expiration', 'get_expired', 'access_token', 'refresh_token', 'created',
        'modified')
    readonly_fields = fields
    list_filter = ('client', 'created', 'modified')
    actions = ['refresh_tokens']

    def has_add_permission(self, request):
        return False

    def refresh_tokens(self, request, queryset):
        for access_token in queryset:
            access_token.do_refresh_token()
        msg = ugettext("Selected tokens refreshed.")
        self.message_user(request, msg, messages.INFO)
    refresh_tokens.short_description = _("Refresh selected tokens")

    def get_expiration(self, obj):
        return localize(obj.expiration())
    get_expiration.short_description = _("Expiration")

    def get_expired(self, obj):
        return obj.is_expired()
    get_expired.short_description = _("Expired")
    get_expired.boolean = True


admin.site.register(models.Client, ClientAdmin)
admin.site.register(models.AccessToken, TokenAdmin)



# -*- coding: utf-8 -*-
from django.conf.urls import url
from django.views.generic import TemplateView

from . import views


app_name = 'oauth_clients'
urlpatterns = [
    url(
        regex="^Client/~create/$",
        view=views.ClientCreateView.as_view(),
        name='Client_create',
    ),
    url(
        regex="^Client/(?P<pk>\d+)/~delete/$",
        view=views.ClientDeleteView.as_view(),
        name='Client_delete',
    ),
    url(
        regex="^Client/(?P<pk>\d+)/$",
        view=views.ClientDetailView.as_view(),
        name='Client_detail',
    ),
    url(
        regex="^Client/(?P<pk>\d+)/~update/$",
        view=views.ClientUpdateView.as_view(),
        name='Client_update',
    ),
    url(
        regex="^Client/$",
        view=views.ClientListView.as_view(),
        name='Client_list',
    ),
	url(
        regex="^AccessToken/~create/$",
        view=views.AccessTokenCreateView.as_view(),
        name='AccessToken_create',
    ),
    url(
        regex="^AccessToken/(?P<pk>\d+)/~delete/$",
        view=views.AccessTokenDeleteView.as_view(),
        name='AccessToken_delete',
    ),
    url(
        regex="^AccessToken/(?P<pk>\d+)/$",
        view=views.AccessTokenDetailView.as_view(),
        name='AccessToken_detail',
    ),
    url(
        regex="^AccessToken/(?P<pk>\d+)/~update/$",
        view=views.AccessTokenUpdateView.as_view(),
        name='AccessToken_update',
    ),
    url(
        regex="^AccessToken/$",
        view=views.AccessTokenListView.as_view(),
        name='AccessToken_list',
    ),
	]



# Generated by Django 2.1.2 on 2018-11-26 21:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oauth_clients', '0002_client_scope'),
    ]

    operations = [
        migrations.AlterField(
            model_name='client',
            name='scope',
            field=models.CharField(blank=True, default='read', max_length=255, verbose_name='Scope'),
        ),
        migrations.AlterField(
            model_name='client',
            name='token_endpoint',
            field=models.URLField(blank=True, default='', max_length=255, verbose_name='Token endpoint'),
        ),
    ]



# -*- coding: utf-8 -*-
from django.views.generic import (
    CreateView,
    DeleteView,
    DetailView,
    UpdateView,
    ListView
)

from .models import (
	Client,
	AccessToken,
)


class ClientCreateView(CreateView):

    model = Client


class ClientDeleteView(DeleteView):

    model = Client


class ClientDetailView(DetailView):

    model = Client


class ClientUpdateView(UpdateView):

    model = Client


class ClientListView(ListView):

    model = Client


class AccessTokenCreateView(CreateView):

    model = AccessToken


class AccessTokenDeleteView(DeleteView):

    model = AccessToken


class AccessTokenDetailView(DetailView):

    model = AccessToken


class AccessTokenUpdateView(UpdateView):

    model = AccessToken


class AccessTokenListView(ListView):

    model = AccessToken




# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-11-21 19:30
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AccessToken',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('token_type', models.CharField(max_length=255, verbose_name='Token type')),
                ('expires_in', models.IntegerField(verbose_name='Expires in')),
                ('access_token', models.CharField(max_length=255, verbose_name='Access token')),
                ('refresh_token', models.CharField(max_length=255, verbose_name='Refresh token')),
            ],
            options={
                'verbose_name': 'Access token',
                'verbose_name_plural': 'Access tokens',
            },
        ),
        migrations.CreateModel(
            name='Client',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('uid', models.UUIDField(default=uuid.uuid4, unique=True, verbose_name='UID')),
                ('name', models.CharField(max_length=100, unique=True, verbose_name='Name')),
                ('client_id', models.CharField(max_length=255, unique=True, verbose_name='Client id')),
                ('client_secret', models.CharField(max_length=255, verbose_name='Client secret')),
                ('authorization_endpoint', models.URLField(max_length=255, verbose_name='Authorization endpoint')),
                ('token_endpoint', models.URLField(blank=True, default=b'', max_length=255, verbose_name='Token endpoint')),
                ('creator', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Oauth2 client',
                'verbose_name_plural': 'Oauth2 clients',
            },
        ),
        migrations.AddField(
            model_name='accesstoken',
            name='client',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='oauth_clients.Client', verbose_name='Client'),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-22 17:32

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('oauth_clients', '0004_auto_20181127_0314'),
    ]

    operations = [
        migrations.CreateModel(
            name='ClientParam',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=40, verbose_name='Name')),
                ('value', models.TextField(blank=True, default='', verbose_name='Value')),
                ('client', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='params', to='oauth_clients.Client', verbose_name='Client')),
            ],
            options={
                'verbose_name': 'Client param',
                'verbose_name_plural': 'Client params',
            },
        ),
    ]



# -*- coding: utf-8 -*-
import uuid
import requests
import datetime
from django.conf import settings
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse
from django.utils.six import python_2_unicode_compatible
from django.utils.six.moves.urllib.parse import urlencode
from model_utils.models import TimeStampedModel


@python_2_unicode_compatible
class Client(TimeStampedModel):
    uid = models.UUIDField(_("UID"), default=uuid.uuid4, unique=True)
    name = models.CharField(_("Name"), max_length=100, unique=True)
    client_id = models.CharField(_("Client id"), max_length=255, unique=True)
    client_secret = models.CharField(_("Client secret"), max_length=255)
    authorization_endpoint = models.URLField(_("Authorization endpoint"), max_length=255)
    token_endpoint = models.URLField(_("Token endpoint"), max_length=255, blank=True, default='')
    creator = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_("User"), on_delete=models.PROTECT)
    scope = models.CharField(_("Scope"), max_length=255, blank=True, default='read')

    class Meta:
        verbose_name = _("Oauth2 client")
        verbose_name_plural = _("Oauth2 clients")

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        if not self.token_endpoint:
            self.token_endpoint = self.authorization_endpoint
        return super(Client, self).save(*args, **kwargs)

    @property
    def session_state_name(self):
        return str(self.uid)

    @property
    def session_redirect_url_name(self):
        return '{}:redirect-url'.format(str(self.uid))

    def complete_authorization_url(self, request=None):
        info = self._meta.app_label, self._meta.model_name
        view_name = 'admin:%s_%s_complete_authorization' % info
        authorization_url = reverse(view_name, args=(self.uid,))
        return request.build_absolute_uri(authorization_url) if request else authorization_url

    def start_authorization_url(self, request, redirect_url, final_redirection=None):
        redirect_url = request.build_absolute_uri(redirect_url)
        state = str(uuid.uuid4())
        params = {
            'response_type': 'code',
            'client_id': self.client_id,
            'redirect_uri': redirect_url,
            'scope': self.scope,
            'state': state,
        }
        for row in self.params.all():
            params[row.name] = row.value
        encoded_params = urlencode(params)
        result = '{}?{}'.format(self.authorization_endpoint, encoded_params)
        request.session[self.session_state_name] = state
        if final_redirection:
            request.session[self.session_redirect_url_name] = final_redirection
        return result

    def complete_authorization(self, request, redirect_url):
        state = request.GET['state']
        if request.session[self.session_state_name] != state:
            raise ValueError("Wrong oauth state.")
        del request.session[self.session_state_name]
        code = request.GET['code']
        token_url = '{}?grant_type=authorization_code&client_id={}&client_secret={}&redirect_uri={}&code={}'
        token_url = token_url.format(self.token_endpoint, self.client_id, self.client_secret, redirect_url, code)
        response = requests.post(token_url)
        data = response.json()
        access_token = AccessToken()
        access_token.client = self
        access_token.token_type = data['token_type']
        access_token.expires_in = data['expires_in']
        access_token.access_token = data['access_token']
        access_token.refresh_token = data.get('refresh_token', '')
        access_token.scope = data.get('scope', self.scope)
        access_token.save()
        return request.session.pop(self.session_redirect_url_name, None)

@python_2_unicode_compatible
class ClientParam(models.Model):
    client = models.ForeignKey(Client, verbose_name=_("Client"),
        related_name='params', on_delete=models.CASCADE)
    name = models.CharField(_("Name"), max_length=40)
    value = models.TextField(_("Value"), blank=True, default='')

    class Meta:
        verbose_name = _("Client param")
        verbose_name_plural = _("Client params")

    def __str__(self):
        return self.name


class AccessToken(TimeStampedModel):
    client = models.ForeignKey(Client, verbose_name=_("Client"), on_delete=models.CASCADE)
    user_id = models.CharField(_("User id"), max_length=255, blank=True, default='')
    username = models.CharField(_("Username"), max_length=255, blank=True, default='')
    token_type = models.CharField(_("Token type"), max_length=255)
    expires_in = models.IntegerField(_("Expires in"))
    access_token = models.TextField(_("Access token"))
    refresh_token = models.TextField(_("Refresh token"))
    scope = models.TextField(_("Scope"), blank=True, default='')

    class Meta:
        verbose_name = _("Access token")
        verbose_name_plural = _("Access tokens")

    def do_refresh_token(self):
        url = '{}?grant_type=refresh_token&refresh_token={}&client_id={}&client_secret={}&scope={}'
        url = url.format(self.client.token_endpoint, self.refresh_token, self.client.client_id,
            self.client.client_secret, self.client.scope)
        response = requests.post(url)
        data = response.json()
        self.token_type = data['token_type']
        self.expires_in = data['expires_in']
        self.access_token = data['access_token']
        if 'refresh_token' in data:
            self.refresh_token = data['refresh_token']
        self.save()

    def expiration(self):
        modified = timezone.localtime(self.modified)
        delta = datetime.timedelta(seconds=self.expires_in)
        return modified + delta

    def is_expired(self):
        return timezone.now() > self.expiration()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_dj-oauth-clients
------------

Tests for `dj-oauth-clients` models module.
"""

from django.test import TestCase

from oauth_clients import models


class TestOauth_clients(TestCase):

    def setUp(self):
        pass

    def test_something(self):
        pass

    def tearDown(self):
        pass



# Generated by Django 2.1.2 on 2018-11-27 03:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oauth_clients', '0003_auto_20181126_2148'),
    ]

    operations = [
        migrations.AddField(
            model_name='accesstoken',
            name='user_id',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='User id'),
        ),
        migrations.AddField(
            model_name='accesstoken',
            name='username',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='Username'),
        ),
    ]
